using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace SpaceTransformationExample
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {

        public const int SCREEN_WIDTH = 1080;
        public const int SCREEN_HEIGHT = 720;
        public const int NUM_STARS = 20;
        public const int NUM_OBJECTS = 30;
        public const float SPEED = 60;

        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        WorldObject[] mObjects;

        Camera mCamera;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
            graphics.PreferredBackBufferHeight = SCREEN_HEIGHT;
            graphics.PreferredBackBufferWidth = SCREEN_WIDTH;
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            Random r = new Random();

            spriteBatch = new SpriteBatch(GraphicsDevice);
            mCamera = new Camera(spriteBatch);

            String[] objects = { "collision1", "collision2", "collision3" };

            mObjects = new WorldObject[NUM_OBJECTS];
            for (int i = 0; i < NUM_OBJECTS; i++)
            {
                mObjects[i] = new WorldObject(Content.Load<Texture2D>(objects[i % objects.Length]), new Vector2((float) r.NextDouble()*SCREEN_WIDTH, (float) r.NextDouble()*SCREEN_HEIGHT));
                mObjects[i].velocity = new Vector2((float) (0.5 - r.NextDouble()) * SPEED, (float) (0.5 - r.NextDouble())*SPEED);
            }

        /// Load up images
        
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            foreach (WorldObject w in mObjects)
            {
                w.color = Color.White;
                w.Update(gameTime);
            }
            for (int i = 0; i < mObjects.Length; i++)
            {
                for (int j = i + 1; j < mObjects.Length; j++)
                {
                    if (mObjects[i].Collides(mObjects[j]))
                    {
                        mObjects[i].color = Color.Red;
                        mObjects[j].color = Color.Red;
                    }
                }
            }

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.Black);

            mCamera.Begin();

            foreach (WorldObject w in mObjects)
            {
                mCamera.Draw(w);
            }
            mCamera.End();

            // TODO: Add your drawing code here

            base.Draw(gameTime);
        }
    }
}
